' FilmToArchive.bas for FilmStar DESIGN
' Copyright 2000-2015 FTG Software Associates

' This program converts FILM files to FILM Archive files.
' Data files MUST be backed up before using this program!!!!!

' The following files should be copied to DataDir$:
'    *.axw, *.flw, *.inw, *.trw, *.vaw

' The following files should be copied to DesignDir$:
'    *.dew

' Path information can be removed. That is, lines in the FILM file
' like "c:\winfilm\mydata\oxides.inw" become "oxides.inw". Path
' information was inserted if the file was stored with the "Save in
' any directory" option in File...Configuration...Directories.

Option Explicit
DefLng i-N

 Private Type FILETIME
     dwLowDate  As Long
     dwHighDate As Long
 End Type

 Private Type SYSTEMTIME
     wYear      As Integer
     wMonth     As Integer
     wDayOfWeek As Integer
     wDay       As Integer
     wHour      As Integer
     wMinute    As Integer
     wSecond    As Integer
     wMillisecs As Integer
 End Type

Private Const OPEN_EXISTING = 3
Private Const FILE_SHARE_READ = &H1
Private Const FILE_SHARE_WRITE = &H2
Private Const GENERIC_WRITE = &H40000000

Private Declare Function CreateFile Lib "kernel32" Alias _
   "CreateFileA" (ByVal lpFileName As String, _
   ByVal dwDesiredAccess As Long, _
   ByVal dwShareMode As Long, _
   ByVal lpSecurityAttributes As Long, _
   ByVal dwCreationDisposition As Long, _
   ByVal dwFlagsAndAttributes As Long, _
   ByVal hTemplateFile As Long) _
   As Long

Private Declare Function LocalFileTimeToFileTime Lib _
     "kernel32" (lpLocalFileTime As FILETIME, _
      lpFileTime As FILETIME) As Long

Private Declare Function SetFileTime Lib "kernel32" _
   (ByVal hFile As Long, lpCreateTime As FILETIME, _
    ByVal NullP2 As Long, lpLastWriteTime _
    As FILETIME) As Long

Private Declare Function SystemTimeToFileTime Lib _
    "kernel32" (lpSystemTime As SYSTEMTIME, lpFileTime _
    As FILETIME) As Long

Private Declare Function CloseHandle Lib "kernel32" _
   (ByVal hObject As Long) As Long

Private FilmDate As Date

' CHANGE DIRECTORIES AS REQUIRED
' VERIFY PATHS IN FILES..CONFIGURATION..DIRECTORIES
Const DataPth$ = "c:\winfilm\data"
Const DesignPth$ = "c:\winfilm\designs"

Sub Main
    Const t$ = "FILM to FILM Archive"
    If MsgBox("This program converts FILM files to FILM Archive files. Not sure you want to do that? Then click <Cancel>.", vbOkCancel + vbInformation + vbDefaultButton2, t$) = vbOK Then
        If MsgBox("Are all design (.dew) and data (.axw, .flw, .inw, .trw, .vaw) files backed-up?", vbYesNo + vbQuestion, t$) = vbNo Then
            MsgBox "Back-up files and re-run program", vbOkOnly + vbInformation, t$
            End
        End If
        If MsgBox("Are there design (.dew) and data (.axw, .inw, .trw, .vaw) files in directories other than" & vbLf & vbLf & DataPth$ _
          & "  " & DesignPth$ & "?", vbYesNo + vbQuestion + vbDefaultButton2, t$) = vbYes Then
            MsgBox "Copy design and data files to default directories and re-run program", vbOkOnly + vbInformation, t$
            End
        Else
            If MsgBox ("Non-default paths will be removed from FILM files." & vbLf & "Click <Cancel> if this step is unnecessary.", vbOkCancel + vbInformation, t$) = vbOK Then
                StatusLabel "Removing non-default paths - please wait"
                RemoveDirInfo
            End If
        End If
        If MsgBox ("Create FILM Archive files?", vbQuestion + vbYesNo,"FILM to FILM Archive") = vbYes Then
            StatusLabel "Creating FILM Archive files - please wait"
            MakeArchive
        End If
    End If
End Sub

Sub RemoveDirInfo
    Dim i, k, f$, p$, s$(0 To 6)
    f$ = Dir$(DataPth & "\*.flw")
    While f$ <> ""
        Open DataPth$ & "\" & f$ For Input As #1
      ' i 0 description 1 graph axes  2 film indices  3 design  4 targets  5 variables  6 FILM
        For i=0 To 6
            Input #1, s$(i)
        Next i
        Close #1
        If InStr(s$(6), "\") > 0  Then
            On Error Resume Next    ' in case backups already created
            Name DataPth$ & "\" & f$ As DataPth$ & "\" & Left$(f$,Len(f$)-4) & ".bak"   ' rename as backup
            On Error GoTo 0
            For i= 1 To 5
                GetFilePath s$(i), p$, s$(i) ' remove path names
            Next i
            Open DataPth$ & "\" & f$ For Output As #1   ' re-store files
            For i = 0 To 6
                Write #1, s$(i)
            Next i
            Close #1
        End If
        f$ = Dir$()
    Wend
End Sub

Sub MakeArchive
    Dim i, k, d$, f$, p$, fName$, s$(0 To 5), ext$(1 To 5)
    ext$(1) = ".axw"
    ext$(2) = ".inw"
    ext$(3) = ".dew"
    ext$(4) = ".trw"
    ext$(5) = ".vaw"
    f$ = Dir$(DataPth & "\*.flw")
    While f$ <> ""
    	On Error GoTo NextFile
        FilmDate = FileDateTime(DataPth$ & "\" & f$)
        Open DataPth$ & "\" & f$ For Input As #1
      ' i 0 description 1 graph axes  2 film indices  3 design  4 targets  5 variables
        For i=0 To 5
            Input #1, s$(i)
        Next i
        Close #1
        For i = 1 To 5   ' check for any missing files
            If s$(i) <> "" Then
                GetFilePath s$(i), p$, fName$
                If p$ = "" Then
                    If i=3 Then p$ = DesignPth$ Else p$ = DataPth$
                End If
                If Not(FileExists(p$ & "\" & s$(i) & ext$(i))) Then
                    GoTo NextFile
                End If
            End If
        Next i
        FileOpen DataPth$ & "\" & f$    '.flw FILM file
        fName$ = Left$(f$, Len(f$)-4)   ' name without extension
        f$ = DesignPth$ & "\" & fName$ & ".faw" '.faw FILM Archive file
        k = 0
        Do While FileExists(f$) ' rename any files with same name
            k = k + 1
            f$ = DesignPth$ & "\" & fName$ & "(" & CStr(k) & ").faw" '.faw FILM Archive fie
        Loop
        If Trim$(s$(0)) <> "" Then FileDescription(8) = s$(0)
        StatusLabel "Saving " & f$
        FileSave f$
        SetFileDateTime f$, CStr(FilmDate)
NextFile:
		Close
        f$ = Dir$()
    Wend
End Sub

Sub GetFilePath(ByVal f$, path$, fName$)
    Dim k, r$
    k = InStr(f$, "\")
    If k = 0 Then
        k = InStr(f$, ":")
        If k = 0 Then
            path$ = ""
            fName$ = f$
        Else
            path$ = Mid(f$, 1, 2)
            fName$ = Mid$(f$, 3)
        End If
    Else
        r$ = StrReverse(f$)
        k = Len(f$) - InStr(r$, "\") + 1
        path$ = Mid$(f$, 1, k - 1)
        fName$ = Mid$(f$, k + 1)
    End If
End Sub

Function FileExists(ByVal f$) As Boolean
    On Error GoTo NoFile
    FileExists = (FileLen(f$) > 0)
    Exit Function
NoFile:
    FileExists = False
    Exit Function
End Function

Public Function SetFileDateTime(ByVal FileName As String, ByVal TheDate As String) As Boolean
' http://www.freevbcode.com/ShowCode.asp?ID=1335
'************************************************
'PURPOSE:    Set File Date (and optionally time)
'            for a given file)

'PARAMETERS: TheDate -- Date to Set File's Modified Date/Time
'            FileName -- The File Name

'Returns:    True if successful, false otherwise
'************************************************
Dim lFileHnd As Long
Dim lRet As Long

Dim typFileTime As FILETIME
Dim typLocalTime As FILETIME
Dim typSystemTime As SYSTEMTIME

If Not FileExists(FileName) Then Exit Function
If Not IsDate(TheDate) Then Exit Function

With typSystemTime
    .wYear = Year(TheDate)
    .wMonth = Month(TheDate)
    .wDay = Day(TheDate)
    .wDayOfWeek = Weekday(TheDate) - 1
    .wHour = Hour(TheDate)
    .wMinute = Minute(TheDate)
    .wSecond = Second(TheDate)
End With

lRet = SystemTimeToFileTime(typSystemTime, typLocalTime)
lRet = LocalFileTimeToFileTime(typLocalTime, typFileTime)

lFileHnd = CreateFile(FileName, GENERIC_WRITE, _
    FILE_SHARE_READ Or FILE_SHARE_WRITE, 0&, _
    OPEN_EXISTING, 0, 0)

lRet = SetFileTime(lFileHnd, typFileTime, 0&, typFileTime)

CloseHandle lFileHnd
SetFileDateTime = lRet > 0

End Function
